local L = AceLibrary("AceLocale-2.2"):new("Enigma")
-- config menu options--
local options = { 
 type='group',
 args = {
  key = {
   type = 'text',
   name = L["Key"],
   desc = L["The encryption key (case sensitive)."],
   usage = L["<Key>"],
   get = "GetKey",
   set = "SetKey",
  },
  channelnumber = {
   type = 'range',
   name = L["Channelnumber"],
   desc = L["Which channel should be encrypted? oh and please don\'t use general or trade."],
   usage = L["<Channelnumber>"],
   min=1,
   max=10,
   get = "GetChannum",
   set = "SetChannum",
  },
 },
}
-- end config --

-- basics --
Enigma = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
Enigma:RegisterChatCommand({"/enigma"}, options)
Enigma:RegisterDB("EnigmaDB", "EnigmaDBPC")
Enigma:RegisterDefaults("char", {
 key = "deadbeef",
 channum = 9
 }
) 
-- end basics --

function Enigma:OnInitialize()
end

function Enigma:OnEnable()
 self:Print(L["Warning: Enigma is still beta, so use it at your own risk!"])
 self:Hook(DEFAULT_CHAT_FRAME, "AddMessage", true)
 self:Hook(ChatFrame2, "AddMessage", true)
 self:Hook(ChatFrame3, "AddMessage", true)
 self:Hook(ChatFrame4, "AddMessage", true)
 self:Hook(ChatFrame5, "AddMessage", true)
 self:Hook(ChatFrame6, "AddMessage", true)
 self:Hook(ChatFrame7, "AddMessage", true)
 self:Hook("SendChatMessage", true)
end

function Enigma:OnDisable()
 self:UnhookAll() 
 self:Print("Disabled")
end

function Enigma:GetKey()
 return self.db.char.key
end

function Enigma:SetKey(newValue)
 self.db.char.key = newValue
end

function Enigma:GetChannum()
 return self.db.char.channum
end

function Enigma:SetChannum(newValue)
 self.db.char.channum = newValue
end

function Enigma:AddMessage(frame, text, r, g, b, id)
 if string.find(text, "Enigmatic") == nil then 
  return self.hooks[frame].AddMessage(frame, text, r, g, b, id) --not enigma, so lets just move along
 else 
  local enigmastart = nil
  local enigmastop = nil
  enigmastart, enigmastop = string.find(text, "Enigmatic") --enigma, so let's do our magic
  return self.hooks[frame].AddMessage(frame, "Decrypted: " .. string.sub(text, 1, enigmastart - 1) .. Enigma:Decrypt(string.sub(text, enigmastop + 1)), r, g, b, id) 
 end
end

function Enigma:SendChatMessage(text, type, language, target)
 if target ~= Enigma:GetChannum() then
  return self.hooks.SendChatMessage(text, type, language, target)
 else
  return self.hooks.SendChatMessage("Enigmatic" .. Enigma:Encrypt(text), type, language, target)
 end 
end

function Enigma:Decrypt(message)
 local plain = message
 local out = ""
 local key = Enigma:GetKey()
 local plainlength = string.len(plain)
 local keylength = string.len(key)
 local plaincount = 1
 local keycount = 1
 local temp = 0
 repeat 
  if keycount > keylength then keycount = 1 end
  temp = (((strbyte(plain, plaincount) - 32) - (strbyte(key, keycount) - 32)) % 92) + 32 --it's a hack alright: to understand it, just look at your utf-8 table
  out = out .. strchar(temp)
  keycount = keycount + 1
  plaincount = plaincount + 1
 until plaincount > plainlength 
 return out
end

function Enigma:Encrypt(message)
 local plain = message
 local out = ""
 local key = Enigma:GetKey()
 local plainlength = string.len(plain)
 local keylength = string.len(key)
 local plaincount = 1
 local keycount = 1
 local temp = 0
 repeat 
  if keycount > keylength then keycount = 1 end
  temp = (((strbyte(plain, plaincount) - 32) + (strbyte(key, keycount) - 32)) %92 ) + 32
  out = out .. strchar(temp)
  keycount = keycount + 1
  plaincount = plaincount + 1
 until plaincount > plainlength 
 return out
end